/*
 * Occurrences.cpp
 *
 *  Created on: 9. 2. 2014
 *	  Author: Jan Travnicek
 */

#include "Occurrences.h"
#include "Run.h"
#include <exception/AlibException.h>
#include <automaton/FSM/DFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/TA/DFTA.h>
#include <automaton/PDA/InputDrivenDPDA.h>
#include <automaton/PDA/VisiblyPushdownDPDA.h>
#include <automaton/PDA/DPDA.h>

#include <deque>

namespace automaton {

namespace run {

std::set < unsigned > Occurrences::occurrences ( const automaton::Automaton & automaton, const alib::Object & object ) {
	return getInstance ( ).dispatch ( automaton.getData ( ), object.getData ( ) );
}

std::set < unsigned > Occurrences::occurrences ( const automaton::Automaton & automaton, const string::LinearString & string ) {
	return getInstance ( ).dispatch ( automaton.getData ( ), string );
}

std::set < unsigned > Occurrences::occurrences ( const automaton::Automaton & automaton, const tree::RankedTree & tree ) {
	return getInstance ( ).dispatch ( automaton.getData ( ), tree );
}

std::set < unsigned > Occurrences::occurrences ( const automaton::DFA & automaton, const string::LinearString & string ) {
	std::tuple < bool, automaton::State, std::set < unsigned > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

auto OccurrencesDFALinearString = Occurrences::RegistratorWrapper < std::set < unsigned >, automaton::DFA, string::LinearString > ( Occurrences::getInstance ( ), Occurrences::occurrences );

std::set < unsigned > Occurrences::occurrences ( const automaton::DFTA & automaton, const tree::RankedTree & tree ) {
	std::tuple < bool, automaton::State, std::set < unsigned > > res = Run::calculateState ( automaton, tree );

	return std::get < 2 > ( res );
}

auto OccurrencesDFTARankedTree = Occurrences::RegistratorWrapper < std::set < unsigned >, automaton::DFTA, tree::RankedTree > ( Occurrences::getInstance ( ), Occurrences::occurrences );

std::set < unsigned > Occurrences::occurrences ( const automaton::InputDrivenDPDA & automaton, const string::LinearString & string ) {
	std::tuple < bool, automaton::State, std::set < unsigned > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

auto OccurrencesInputDrivenDPDALinearString = Occurrences::RegistratorWrapper < std::set < unsigned >, automaton::InputDrivenDPDA, string::LinearString > ( Occurrences::getInstance ( ), Occurrences::occurrences );

std::set < unsigned > Occurrences::occurrences ( const automaton::VisiblyPushdownDPDA & automaton, const string::LinearString & string ) {
	std::tuple < bool, automaton::State, std::set < unsigned > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

auto OccurrencesVisiblyPushdownDPDALinearString = Occurrences::RegistratorWrapper < std::set < unsigned >, automaton::VisiblyPushdownDPDA, string::LinearString > ( Occurrences::getInstance ( ), Occurrences::occurrences );

std::set < unsigned > Occurrences::occurrences ( const automaton::DPDA & automaton, const string::LinearString & string ) {
	std::tuple < bool, automaton::State, std::set < unsigned > > res = Run::calculateState ( automaton, string );

	return std::get < 2 > ( res );
}

auto OccurrencesDPDALinearString = Occurrences::RegistratorWrapper < std::set < unsigned >, automaton::DPDA, string::LinearString > ( Occurrences::getInstance ( ), Occurrences::occurrences );

} /* namespace run */

} /* namespace automaton */
