/*
 * Result.cpp
 *
 *  Created on: 9. 2. 2014
 *	  Author: Jan Travnicek
 */

#include "Result.h"
#include "Run.h"
#include <exception/AlibException.h>
#include <automaton/FSM/DFA.h>
#include <automaton/TA/DFTA.h>
#include <automaton/PDA/InputDrivenDPDA.h>
#include <automaton/PDA/VisiblyPushdownDPDA.h>
#include <automaton/PDA/DPDA.h>
#include <label/LabelSetLabel.h>

#include <deque>

namespace automaton {

namespace run {

label::Label Result::result ( const automaton::Automaton & automaton, const alib::Object & object ) {
	return getInstance ( ).dispatch ( automaton.getData ( ), object.getData ( ) );
}

label::Label Result::result ( const automaton::DFA & automaton, const string::LinearString & string ) {
	std::tuple < bool, automaton::State, std::set < unsigned > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res ).getName ( );

	return label::labelFrom ( "fail" );
}

auto ResultDFALinearString = Result::RegistratorWrapper < label::Label, automaton::DFA, string::LinearString > ( Result::getInstance ( ), Result::result );

label::Label Result::result ( const automaton::DFTA & automaton, const tree::RankedTree & tree ) {
	std::tuple < bool, automaton::State, std::set < unsigned > > res = Run::calculateState ( automaton, tree );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res ).getName ( );

	return label::labelFrom ( "fail" );
}

auto ResultDFTARankedTree = Result::RegistratorWrapper < label::Label, automaton::DFTA, tree::RankedTree > ( Result::getInstance ( ), Result::result );

label::Label Result::result ( const automaton::InputDrivenDPDA & automaton, const string::LinearString & string ) {
	std::tuple < bool, automaton::State, std::set < unsigned > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res ).getName ( );

	return label::labelFrom ( "fail" );
}

auto ResultInputDrivenDPDALinearString = Result::RegistratorWrapper < label::Label, automaton::InputDrivenDPDA, string::LinearString > ( Result::getInstance ( ), Result::result );

label::Label Result::result ( const automaton::VisiblyPushdownDPDA & automaton, const string::LinearString & string ) {
	std::tuple < bool, automaton::State, std::set < unsigned > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res ).getName ( );

	return label::labelFrom ( "fail" );
}

auto ResultVisiblyPushdownDPDALinearString = Result::RegistratorWrapper < label::Label, automaton::VisiblyPushdownDPDA, string::LinearString > ( Result::getInstance ( ), Result::result );

label::Label Result::result ( const automaton::DPDA & automaton, const string::LinearString & string ) {
	std::tuple < bool, automaton::State, std::set < unsigned > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res ).getName ( );

	return label::labelFrom ( "fail" );
}

auto ResultDPDALinearString = Result::RegistratorWrapper < label::Label, automaton::DPDA, string::LinearString > ( Result::getInstance ( ), Result::result );

} /* namespace run */

} /* namespace automaton */
